import sys
import os

from aigpy import pathHelper
# from aigpy import versionHelper
from aigpy.cmdHelper import myinput

import tidal_dl.tidal as tidal
from tidal_dl.tidal import TidalConfig
from tidal_dl.tidal import TidalAccount
from tidal_dl.download import Download

def logIn():
    print("----------------LogIn------------------")
    username = myinput("username:")
    password = myinput("password:")
    account = TidalAccount(username, password)
    if account.errmsg != "":
        print(account.errmsg)
        return

    cf = TidalConfig()
    cf.set_account(username, password, account.session_id, account.country_code)
    return


def setting():
    cf = TidalConfig()
    print("----------------Setting----------------")
    while True:
        outputdir = myinput("outputdir:")
        if os.path.isdir(outputdir) == False:
            print("[Err]Path is Err!")
            continue
        break
    while True:
        quality = myinput("quality  :")
        if cf.valid_quality(quality) == False:
            print("[Err]Quality Err,Only Have " + str(tidal.QUALITY))
            continue
        break

    cf.set_outputdir(outputdir)
    cf.set_quality(quality)

    pathHelper.mkdirs(outputdir + "\\Album\\")
    pathHelper.mkdirs(outputdir + "\\Track\\")
    pathHelper.mkdirs(outputdir + "\\Playlist\\")
    pathHelper.mkdirs(outputdir + "\\Video\\")
    return

def main(argv=None):
    cf = TidalConfig()
    print(tidal.LOG)
    print("====================Tidal-dl========================")
    print("OutputDir    :\t" + __file__)
    print("OutputDir    :\t" + cf.outputdir)
    print("SessionID    :\t" + cf.sessionid)
    print("CountryCode  :\t" + cf.countrycode)
    print("SoundQuality :\t" + cf.quality)
    print("====================================================")

    if cf.sessionid == "":
        logIn()

    dl = Download()
    while True:
        print("=====================Choice=========================")
        print(" Enter '0' : Exit")
        print(" Enter '1' : LogIn And Get SessionID.")
        print(" Enter '2' : Setting(OutputDir/Quality).")
        print(" Enter '3' : Download Album.")
        print(" Enter '4' : Download Track.")
        print(" Enter '5' : Download PlayList.")
        print(" Enter '6' : Download Video")
        # print(" Enter '7' : Download By File")
        print("====================================================")
        choice = myinput("Choice:")
        if choice == '0':
            return
        elif choice == '1':
            logIn()
        elif choice == '2':
            setting()
        elif choice == '3':
            dl.downloadAlbum()
        elif choice == '4':
            dl.downloadTrack()
        elif choice == '5':
            dl.downloadPlaylist()
        elif choice == '6':
            dl.downloadVideo()
        # elif choice == '7':
        #     dl.downloadByFile()

# if __name__ == '__main__':
#     main(sys.argv)


__all__ = ['main', 'tidal', 'download']



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@File    :   ffmpegHelper.py
@Time    :   2018/12/17
@Author  :   Yaron Huang 
@Version :   1.0
@Contact :   yaronhuang@qq.com
@Desc    :   
'''

import os
import re
import sys
import shutil
import subprocess

import aigpy.netHelper as netHelper
import aigpy.pathHelper as pathHelper
import aigpy.threadHelper as threadHelper

from aigpy.progressHelper import ProgressTool

class FFmpegTool(object):
    def __init__(self, threadNum=50):
        self.thread        = threadHelper.ThreadTool(threadNum)
        self.waitCount     = 0
        self.completeCount = 0
        self.progress      = None
        return
    
    def __thradfunc_dl(self, url, filepath, retrycount):
        check = False
        try:
            while retrycount > 0:
                retrycount = retrycount - 1
                check = netHelper.downloadFile(url, filepath)
                if check:
                    break
        except:
            pass
        self.completeCount = self.completeCount + 1
        if self.progress != None:
            self.progress.step()
        return

    
    def __parseM3u8(self, url):
        content = netHelper.downloadString(url, None)
        pattern = re.compile(r"(?<=http).+?(?=\\n)")
        plist = pattern.findall(str(content))
        urllist = []
        for item in plist:
            urllist.append("http"+item)
        return urllist

    def mergerByM3u8_Multithreading(self, url, filepath, showprogress=False, showshell=False):
        try:
            # Get urllist
            urllist = self.__parseM3u8(url)
            if len(urllist) <= 0:
                return False

            # Creat tmpdir
            path = pathHelper.getDirName(filepath)
            tmpPath = pathHelper.getDiffTmpPathName(path)
            if pathHelper.mkdirs(tmpPath) == False:
                return False
            
            # Progress
            self.progress = None
            if showprogress:
                self.progress = ProgressTool(len(urllist))

            # Download files
            index              = 0
            allpath            = []
            self.waitCount     = len(urllist)
            self.completeCount = 0
            for item in urllist:
                index = index + 1
                path = tmpPath + '\\' + str(index) + ".mp4"
                allpath.append(path)
                if os.path.exists(path):
                    os.remove(path)
                self.thread.start(self.__thradfunc_dl, item, path, 3)
            self.thread.waitAll()
            self.mergerByFiles(allpath, filepath, showshell)
            shutil.rmtree(tmpPath)
            return True
        except:
            return False

    def mergerByM3u8(self, url, filepath, showshell=False):
        try:
            cmd = "ffmpeg -safe 0 -i " + url + " -c copy -bsf:a aac_adtstoasc \"" + filepath + "\""
            if showshell:
                res = subprocess.call(cmd, shell=True)
            else:
                res = subprocess.call(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            if res != 0:
                return False
            return True
        except:
            return False

    def mergerByFiles(self, srcfilepaths, filepath, showshell=False):
        result = True
        tmpfile = filepath + "TMP.txt"
        try:
            with open(tmpfile, 'w') as fd:
                for item in srcfilepaths:
                    fd.write('file \'' + item + '\'\n')

            cmd = "ffmpeg -f concat -safe 0 -i \"" + tmpfile + "\" -c copy \"" + filepath + "\""
            if showshell:
                res = subprocess.call(cmd, shell=True)
            else:
                res = subprocess.call(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            if res != 0:
                result = False
        except:
            result = False

        if os.access(tmpfile,0):
            os.remove(tmpfile)
        return result



from setuptools import setup, find_packages

setup(
    name = 'tidal-dl',
    version='2018.12.28.0',
    license = "MIT Licence",
    description = "Tidal Music Download.",

    author = 'YaronH',
    author_email = "yaronhuang@qq.com",

    packages = find_packages(),
    include_package_data = True,
    platforms = "any",
    install_requires=["aigpy>=1.0.32", "requests", "ffmpeg", "futures"],

    entry_points={'console_scripts': [
        'tidal-dl = tidal_dl:main', ]}
)
